/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.pixlib.quick.document.license.type
{
	import net.pixlib.log.PXStringifier;
	import net.pixlib.quick.document.license.PXLicense;
	import net.pixlib.quick.document.license.PXLicenseEvent;

	import com.adobe.licensing.LicenseManager;

	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.events.EventDispatcher;


	/**
	 * Dispatched when license is checked (or canceled).
	 * 
	 * @eventType net.pixlib.quick.document.license.LicenseEvent.onLicenseCheckEVENT
	 */
	[Event(name="onLicenseCheck", type="net.pixlib.quick.document.license.PXLicenseEvent")]

	/**
	 * The InMarketLicense class implements simple License checking for 
	 * Adobe AIR InMarket application.
	 * 
	 * @example Using standalone implementation.
	 * <listing>
	 * 
	 * public function check() : void
	 * {
	 * 	var license : InMarketLicense = new InMarketLicense(this, "YOUR_UID");
	 * 	license.addEventListener(LicenseEvent.onLicenseCheckEVENT, _onLicense);
	 * 	license.authorize();
	 * }
	 * 
	 * public function _onLicense(event : LicenseEvent) : void
	 * {
	 * 	trace(event.license.full);
	 * }
	 * </listing>
	 * 
	 * @example Using PixQuick engine
	 * <listing>
	 * 
	 * package
	 * {
	 * 	import net.pixlib.quick.document.DocumentClass;
	 * 	
	 * 	public class MainDocument extends DocumentClass
	 * 	{
	 * 		public function MainDocument()
	 * 		{
	 * 			super(null, new InMarketLicense("YOUR_UID"));
	 * 		}
	 * 	}
	 * </listing>
	 * 
	 * @see net.pixlib.quick.document.license.License
	 * @see net.pixlib.quick.document.license.LicenseManager
	 * 
	 * @langversion 3.0
	 * @playerversion Flash 10
	 * @playerversion AIR 2.5
	 * 
	 * @author Romain Ecarnot
	 */
	final public class PXInMarketLicense extends EventDispatcher implements PXLicense
	{
		// --------------------------------------------------------------------
		// Private properties
		// --------------------------------------------------------------------

		/**
		 * @private
		 */
		private var _uid : String;

		/**
		 * @private
		 */
		private var _updateMode : Boolean;

		/**
		 * @private
		 */
		private var _debugMode : Boolean;

		/**
		 * @private
		 */
		private var _manager : LicenseManager;

		/**
		 * @private
		 */
		private var _owner : DisplayObject;


		// --------------------------------------------------------------------
		// Public properties
		// --------------------------------------------------------------------

		/**
		 * @inheritDoc
		 */
		public function get full() : Boolean
		{
			return _manager.full;
		}

		/**
		 * @inheritDoc
		 */
		public function get ttl() : Number
		{
			return _manager.ttl;
		}

		/**
		 * @inheritDoc
		 */
		public function get url() : String
		{
			return _manager.url;
		}


		// --------------------------------------------------------------------
		// Public API
		// --------------------------------------------------------------------

		/**
		 * Creates new instance.
		 * 
		 * @langversion 3.0
		 * @playerversion Flash 10
		 * @playerversion AIR 2.5
		 */
		public function PXInMarketLicense(owner : DisplayObject, uid : String, updateMode : Boolean = true, debugMode : Boolean = false)
		{
			_owner = owner;
			_uid = uid;
			_updateMode = updateMode;
			_debugMode = debugMode;

			_manager = new LicenseManager();
		}

		/**
		 * @inheritDoc
		 */
		public function authorize() : void
		{
			_manager.addEventListener(LicenseManager.EVENT_COMPLETE, _onComplete);
			_manager.addEventListener(LicenseManager.EVENT_EXITING, _onExit);
			_manager.checkLicense(_owner, _uid, _updateMode, _debugMode);
		}

		/**
		 * @inheritDoc
		 */
		override public function toString() : String
		{
			return PXStringifier.process(this) + "[" + full + "]";
		}


		// --------------------------------------------------------------------
		// Private methods
		// --------------------------------------------------------------------

		/**
		 * @private
		 */
		private function _onExit(event : Event) : void
		{
			event.stopImmediatePropagation();
			_releaseListeners();

			dispatchEvent(new PXLicenseEvent(PXLicenseEvent.onLicenseCheckEVENT, this));
		}

		/**
		 * @private
		 */
		private function _onComplete(event : Event) : void
		{
			event.stopImmediatePropagation();
			_releaseListeners();

			dispatchEvent(new PXLicenseEvent(PXLicenseEvent.onLicenseCheckEVENT, this));
		}

		/**
		 * @private
		 */
		private function _releaseListeners() : void
		{
			_manager.removeEventListener(LicenseManager.EVENT_COMPLETE, _onComplete);
			_manager.removeEventListener(LicenseManager.EVENT_EXITING, _onExit);
		}
	}
}
